var LOGFILE_NAME = "event-push-notification.log";
var lInfo = new Logger(LOGFILE_NAME);
var lWarn = new Logger(LOGFILE_NAME);

var Utf8 = {
		 
    // public method for url encoding
    encode : function (string) {
        string = string.replace(/\r\n/g,"\n");
        var utftext = "";
 
        for (var n = 0; n < string.length; n++) {
 
            var c = string.charCodeAt(n);
 
            if (c < 128) {
                utftext += String.fromCharCode(c);
            }
            else if((c > 127) && (c < 2048)) {
                utftext += String.fromCharCode((c >> 6) | 192);
                utftext += String.fromCharCode((c & 63) | 128);
            }
            else {
                utftext += String.fromCharCode((c >> 12) | 224);
                utftext += String.fromCharCode(((c >> 6) & 63) | 128);
                utftext += String.fromCharCode((c & 63) | 128);
            }
 
        }
 
        return utftext;
    },
 
    // public method for url decoding
    decode : function (utftext) {
        var string = "";
        var i = 0;
        var c, c2, c3;
        c = c2 = c3 = 0;
        while ( i < utftext.length ) {
 
            c = utftext.charCodeAt(i);
 
            if (c < 128) {
                string += String.fromCharCode(c);
                i++;
            }
            else if((c > 191) && (c < 224)) {
                c2 = utftext.charCodeAt(i+1);
                string += String.fromCharCode(((c & 31) << 6) | (c2 & 63));
                i += 2;
            }
            else {
                c2 = utftext.charCodeAt(i+1);
                c3 = utftext.charCodeAt(i+2);
                string += String.fromCharCode(((c & 15) << 12) | ((c2 & 63) << 6) | (c3 & 63));
                i += 3;
            }
 
        }
 
        return string;
    }
 
};

function storeSubParameter(path,value)
{
	Property.setProperty('values/' + path, value);
	Property.setFlag('values/' + path, 'ARCHIVE', true);
}

/**
 * Store a dsuid as string property parameter.
 * check weather the provided value is a string. In case it's a number it is converted to string, 
 * if it is undefined it tries to convert dsid to dsuid, in any other cases it is discarded
 * @param path [in] property path
 * @param dsuid [in] value to store should be string
 * @param dsid [in] dsid value to be converted in dsuid if not available
 * @returns {Boolean} returns false if value could not be converted
 */
function checkAndStoreDSUIDParameter(path, dsuid, dsid) {
    try 
    {
        lInfo.logln('store dsuid at |' + path + "| value: |" + dsuid + "|" );
        var successCode = false;
        var stringValue;
        if (typeof(dsuid) === 'string') {
            Property.setProperty('values/' + path, dsuid);
            Property.setFlag('values/' + path, 'ARCHIVE', true);
            successCode = true;
        } else {
            if (typeof(dsid) === 'string') {
                stringValue = dSID2dSUID(dsid);
                if (typeof(stringValue) === "string") {
                    Property.setProperty('values/' + path, stringValue);
                    Property.setFlag('values/' + path, 'ARCHIVE', true);
                    successCode = true;
                }                
                lInfo.logln("store converted dsid");
            } else {
                lWarn.logln('Typ-Fehler beim Speichern der Property values/' + path);
            }
        }
        return successCode;
    }
    catch (e)
    {
        lWarn.logln("Exception beim checkAndStoreDSUIDParameter " + e + " Path " + path  + " value " + dsuid);
        return false;
    }        
}

function dSID2dSUID(dSID) {
    try {
        //copy the first 6 bytes (12 characters to the new dsuid
        var dSUID = dSID.slice(0, 12);
        //insert 4 bytes of 0
        dSUID += '00000000';
        //append the remaining 6 bytes (12 chars) of the dsid
        dSUID += dSID.slice(12, 24);
        //finaly add 1 byte of 0
        dSUID += '00';
        return dSUID;
    } catch (e) {
        lWarn.logln('Exception in pushNotification::dSID2dSUID ' + e);
        return '';
    }
}

function storeParameter(path,value)
{
	Property.setProperty(path, value);
	Property.setFlag(path, 'ARCHIVE', true);
}

function checkAndUpdateData() {
    lInfo.logln('checkAndUpdateData');
    var fMigrate=false;
    var basePropertyArray = Property.getNode('values');
    if (basePropertyArray!==null) {
        var subPropArray = basePropertyArray.getChildren();
        for ( var iIndex = 0; iIndex < subPropArray.length; iIndex++) {
            if (subPropArray[iIndex].getChild('triggers')!==null) {
                var triggerArray = subPropArray[iIndex].getChild('triggers').getChildren();
                if (triggerArray!==null) {
                    lInfo.logln('--trigger found ' + iIndex);
                    for ( var jIndex = 0; jIndex < triggerArray.length; jIndex++) {
                        var triggerType = triggerArray[jIndex].getChild('type').getValue();
                        switch (triggerType) {
                            case 'device-msg': //intend fallthrough
                            case 'device-binary-input': //intend fallthrough
                            case 'device-scene': //intend fallthrough
                            case 'device-sensor': {
                                if (triggerArray[jIndex].getChild('dsuid')===null) {
                                    lInfo.logln('---converted to dsuid ' + iIndex);
                                    Property.setProperty(triggerArray[jIndex].getPath() + '/dsuid', dSID2dSUID(triggerArray[jIndex].getChild('dsid').getValue()));
                                    fMigrate = true;
                                }
                            }
                            break;
                            case 'state-change': {
                                var triggerName = triggerArray[jIndex].getChild('name').getValue();
                                var nameArray = triggerName.split('.');
                                if ((nameArray[0] === 'dev') && (nameArray[1].length === 24)) {
                                    var dsuid = dSID2dSUID(nameArray[1]);
                                    triggerName = '';
                                    triggerName = triggerName.concat(nameArray[0],'.', dsuid, '.', nameArray[2]);
                                    Property.setProperty(triggerArray[jIndex].getPath() + '/name', triggerName);  
                                    fMigrate=true;
                                }
                                    
                            }
                            break;
                            default: {}
                        }
                    }
                }
            }
            if (subPropArray[iIndex].getChild('conditions')!=null) {
                var statesNode = subPropArray[iIndex].getChild('conditions').getChild('states');
                if (statesNode != null ) {
                    var statesArray = statesNode.getChildren();
                    for (var jIndex = 0; jIndex < statesArray.length; jIndex++) {
                        var stateName = statesArray[jIndex].getName();
                        var stateValue = statesArray[jIndex].getValue();
                        var nameArray = stateName.split('.');
                        if ((nameArray[0] === 'dev') && (nameArray[1].length === 24)) {
                            var dsuid = dSID2dSUID(nameArray[1]);
                            lInfo.logln('Update from dsid ' + nameArray[1] +  ' to dsuid ' + dsuid);
                            newStateName = '';
                            newStateName = newStateName.concat(nameArray[0],'.', dsuid, '.', nameArray[2]);
                            Property.setProperty(statesNode.getPath() + newStateName, stateValue); 
                            lInfo.logln('set Property ' + statesNode.getPath() + newStateName+' to value '+stateValue);
                            statesNode.removeChild(stateName);
                            lInfo.logln('remove child  ' + stateName);
                            fMigrate=true;
                        }
                    }
                }    
            }

        }
    }
    if (fMigrate) {
        Property.store();
    }
}

function putConditionParameter(idValue,oParameterObject) {
	if (oParameterObject.conditions==null)
		return;
	var sSubpath=idValue + '/conditions/';
	if (oParameterObject.conditions.enabled!=null) {
		if (oParameterObject.conditions.enabled==true)
			storeSubParameter(sSubpath + 'enabled', true);
		else 
			storeSubParameter(sSubpath + 'enabled', false);
	}	
	if (oParameterObject.conditions.weekdays!=null) {
		var sString="";
		for (var iIndex=0;iIndex<oParameterObject.conditions.weekdays.length;iIndex++) {
			var sWeekday=oParameterObject.conditions.weekdays[iIndex];
			switch (sWeekday)
			{
				case 'SU':
					sString += '0,';
				break;
				case 'MO':
					sString += '1,';
				break;
				case 'TU':
					sString += '2,';
				break;
				case 'WE':
					sString += '3,';
				break;
				case 'TH':
					sString += '4,';
				break;
				case 'FR':
					sString += '5,';
				break;
				case 'SA':
					sString += '6,';
				break;
			}	
		}
		if (sString!="") {
			sString=sString.substr(0,sString.length-1);
			storeSubParameter(sSubpath + 'weekdays', sString);
		}
	}	
	if (oParameterObject.conditions.systemState!=null) {
		for (var iIndex=0;iIndex<oParameterObject.conditions.systemState.length;iIndex++){
			var sStateName=oParameterObject.conditions.systemState[iIndex].name;
			var sStateValue=oParameterObject.conditions.systemState[iIndex].value;
			if (sStateValue==='false')
				sStateValue=false;
			if (sStateValue==='true')
				sStateValue=true;
			storeSubParameter(sSubpath + 'states/' + sStateName,sStateValue);
		}
	}	
	if (oParameterObject.conditions.addonStates!=null) {
		for (var sAddonId in oParameterObject.conditions.addonStates) {
			for (var iIndex=0;iIndex<oParameterObject.conditions.addonStates[sAddonId].length;iIndex++){
				var sStateName=oParameterObject.conditions.addonStates[sAddonId][iIndex].name;
				var sStateValue=oParameterObject.conditions.addonStates[sAddonId][iIndex].value;
				if (sStateValue==='false')
					sStateValue=2;
				if (sStateValue==='true')
					sStateValue=1;
				if (sStateValue===false)
					sStateValue=2;
				if (sStateValue===true)
					sStateValue=1;
				storeSubParameter(sSubpath + 'addon-states/' + sAddonId + '/' + sStateName,sStateValue);
			}
		}
	}	
	if (oParameterObject.conditions.zoneState!=null) {
		for (var iIndex=0;iIndex<oParameterObject.conditions.zoneState.length;iIndex++){ 
			var zoneID=oParameterObject.conditions.zoneState[iIndex].zone;
			var groupID=oParameterObject.conditions.zoneState[iIndex].group;
			var sceneID=oParameterObject.conditions.zoneState[iIndex].scene;
			
			storeSubParameter(sSubpath + 'zone-states/' + iIndex + '/zone' , zoneID);
			storeSubParameter(sSubpath + 'zone-states/' + iIndex + '/group' , groupID);
			storeSubParameter(sSubpath + 'zone-states/' + iIndex + '/scene' , sceneID);
		}
	}	
	if (oParameterObject.conditions.timeframe!=null) {
		for (var iIndex=0;iIndex<oParameterObject.conditions.timeframe.length;iIndex++){ 
			var from=oParameterObject.conditions.timeframe[iIndex].start;
			var to=oParameterObject.conditions.timeframe[iIndex].end;
			var sFromBase=from.timeBase;
			var sFromOffset=from.offset;
			var sToBase=to.timeBase;
			var sToOffset=to.offset;
			lInfo.logln('from ' + sFromBase + ' off ' + sFromOffset + ' to  ' + sToBase + ' off ' + sToOffset);
			if (sFromBase=='daily')
				if (sToBase=='daily'){
					storeSubParameter(sSubpath + 'time-start' , Math.floor(parseInt("" + sFromOffset,10)/3600) + ':' + Math.floor(parseInt("" + sFromOffset,10)/60)%60 + ':' + Math.floor(parseInt("" + sFromOffset,10)%60));
					storeSubParameter(sSubpath + 'time-end'  , Math.floor(parseInt("" + sToOffset,10)/3600) + ':' + Math.floor(parseInt("" + sToOffset,10)/60)%60 + ':' + Math.floor(parseInt("" + sToOffset,10)%60));
				}
		}	
	}		
}

function storeValue(sParameterString)
{
	var oParameterObject=JSON.parse(unescape(sParameterString));
	var idValue=oParameterObject.id;
	if (idValue==null)
	{
		idValue=1;
		var oParameterBase=Property.getNode("values");
		if (oParameterBase!=null)
		{
			var oParameterArray=oParameterBase.getChildren();
			if (oParameterArray!=null)
			{	
				for (var iIndex=0;iIndex<oParameterArray.length;iIndex++)
				{
					var iNodeName=parseInt(oParameterArray[iIndex].getName(),10);
					if (idValue<=iNodeName)
						idValue=iNodeName+1;
				}
			}
		}		
	}
	
	var baseNode = Property.getNode('values');
	try {
		if (baseNode!=null)
		    baseNode.removeChild(""+idValue);
	}
	catch (e){}
	
	storeSubParameter( idValue + '/name', Utf8.encode(unescape(oParameterObject.name)));
	storeSubParameter( idValue + '/id', idValue);
	storeSubParameter( idValue + '/message', Utf8.encode(unescape(oParameterObject.message)));
	
	for (var iIndex=0;iIndex<oParameterObject.uuid.length;iIndex++)
		storeSubParameter( idValue + '/uuid/' + iIndex, oParameterObject.uuid[iIndex]);
	for (var iIndex=0;iIndex<oParameterObject.triggers.length;iIndex++)
	{
		var sSubpath=idValue + '/triggers/' + (iIndex + 1) + "/";		
		storeSubParameter(sSubpath + "type" , oParameterObject.triggers[iIndex].type);
		switch(oParameterObject.triggers[iIndex].type) {
            case 'zone-scene':
                storeSubParameter(sSubpath + "zone" , oParameterObject.triggers[iIndex].zone);
                storeSubParameter(sSubpath + "group" , oParameterObject.triggers[iIndex].group);
                storeSubParameter(sSubpath + "scene" , oParameterObject.triggers[iIndex].scene);
                if (oParameterObject.triggers[iIndex].forced!=null)
                	storeSubParameter(sSubpath + "forced" , oParameterObject.triggers[iIndex].forced);
                storeSubParameter(sSubpath + "dsid" , -1);
                break;
            case 'undo-zone-scene':
                storeSubParameter(sSubpath + "zone" , oParameterObject.triggers[iIndex].zone);
                storeSubParameter(sSubpath + "group" , oParameterObject.triggers[iIndex].group);
                storeSubParameter(sSubpath + "scene" , oParameterObject.triggers[iIndex].scene);
                storeSubParameter(sSubpath + "dsid" , -1);
                break;
			case 'device-msg':
				storeSubParameter(sSubpath + "dsid" , oParameterObject.triggers[iIndex].dsid);
				checkAndStoreDSUIDParameter(sSubpath + "dsuid" , oParameterObject.triggers[iIndex].dsuid, oParameterObject.triggers[iIndex].dsid);
				storeSubParameter(sSubpath + "msg" , oParameterObject.triggers[iIndex].msg);
				break;
            case 'device-binary-input':
                storeSubParameter(sSubpath + "dsid" , oParameterObject.triggers[iIndex].dsid);
                checkAndStoreDSUIDParameter(sSubpath + "dsuid" , oParameterObject.triggers[iIndex].dsuid, oParameterObject.triggers[iIndex].dsid);
                storeSubParameter(sSubpath + "index" , oParameterObject.triggers[iIndex].index);
                storeSubParameter(sSubpath + "state" , oParameterObject.triggers[iIndex].state);
                break;

			case 'device-scene':
				storeSubParameter(sSubpath + "dsid" , oParameterObject.triggers[iIndex].dsid);
                checkAndStoreDSUIDParameter(sSubpath + "dsuid" , oParameterObject.triggers[iIndex].dsuid, oParameterObject.triggers[iIndex].dsid);
				storeSubParameter(sSubpath + "scene" , oParameterObject.triggers[iIndex].scene);
                if (oParameterObject.triggers[iIndex].buttonIndex!=null)
                    storeSubParameter(sSubpath + "buttonIndex" , oParameterObject.triggers[iIndex].buttonIndex);
				break;
			case 'custom-event':
				storeSubParameter(sSubpath + "event" , oParameterObject.triggers[iIndex].event);
				break;
            case 'device-sensor':
                storeSubParameter(sSubpath + "dsid" , oParameterObject.triggers[iIndex].dsid);
                checkAndStoreDSUIDParameter(sSubpath + "dsuid" , oParameterObject.triggers[iIndex].dsuid, oParameterObject.triggers[iIndex].dsid);
                storeSubParameter(sSubpath + "eventid" , oParameterObject.triggers[iIndex].eventid);
                break;
            case 'state-change':
                storeSubParameter(sSubpath + "name" , oParameterObject.triggers[iIndex].name);
                storeSubParameter(sSubpath + "state" , oParameterObject.triggers[iIndex].state);
                break;
            case 'addon-state-change':
                storeSubParameter(sSubpath + "addon-id" , oParameterObject.triggers[iIndex].addonId);
                storeSubParameter(sSubpath + "name" , oParameterObject.triggers[iIndex].name);
                storeSubParameter(sSubpath + "state" , oParameterObject.triggers[iIndex].state);
                break;
		}
	}
	putConditionParameter(idValue,oParameterObject);
    if (oParameterObject.metaData!=null)
        storeSubParameter(idValue + '/metaData', oParameterObject.metaData);
    Property.store();
    updateTriggerRegistration(Property.getNode('values/' + idValue));
}

function updateTriggerRegistration(oChildNode){
    var oParameter={actions:'execute'};
    lInfo.logln("EP-Config Set Trigger " + oChildNode.getPath());
    registerTrigger(oChildNode.getPath(),'event-push-notification',oParameter);    
}

function updateAllTrigger() {
    Property.load();
    var baseNode = Property.getNode('values');
    if (baseNode == null) {
        lInfo.logln("EP-Config update all trigger : no entries");     
        return;
    }
    var oArrayNode = baseNode.getChildren();
    for ( var iIndex = 0; iIndex < oArrayNode.length; iIndex++) {
        lInfo.logln("EP-Config Set Trigger" + iIndex);     
        var oChildNode = oArrayNode[iIndex];
        updateTriggerRegistration(oChildNode);
    }
}

function sendPath(oPath) {
	try {
		var sText=oPath.getChild('message').getValue();
		sText=Utf8.decode(sText);
		sText=escape(sText);
		var oTarget=[];
		var oTargetProps=oPath.getChild('uuid').getChildren();
		var fMulti=false;
		for (var iIndex=0;iIndex<oTargetProps.length;iIndex++) {
			if (oTargetProps[iIndex].getValue()=='*')
				fMulti=true;
			else
				oTarget.push(oTargetProps[iIndex].getValue());
			lInfo.logln('Target : ' + oTargetProps[iIndex].getValue());
		}
		if (fMulti) {
			lInfo.logln('Multi Target');
			oTargetProps=Property.getNode('devices').getChildren();
			oTarget=[];
			for (var iIndex=0;iIndex<oTargetProps.length;iIndex++) {
				oTarget.push(oTargetProps[iIndex].getChild('uuid').getValue());
				lInfo.logln('Target : ' + oTargetProps[iIndex].getChild('uuid').getValue());
			}
		}
		lInfo.logln('Text: ' + sText);
		cloudService.sendNotification(oTarget,sText);
	} 
	catch (e) {
		lWarn.logln('Send path error for '+oPath.getPath()+' : ' + e );
	}
}

function fetchDeviceList() {
	try {
		lInfo.logln('start fetchDeviceList');
		if (cloudService.getOSPToken()==null) {
			lInfo.logln('Remote Connectivity not ready! just for 5 minutes');
			var evt = new Event("event-push-notification",{actions:'fetchDeviceList','time':'+300'});			
			evt.raise();	
			return;
		}
		cloudService.fetchNotificationList(function (oList) {
			lInfo.logln('getting Listindex: ' + oList.Response.MobileApps.length);
			Property.getNode('/scripts/event-push-notification').removeChild('devices');
			for (var iIndex=0;iIndex<oList.Response.MobileApps.length;iIndex++) {
				storeParameter('devices/' + iIndex +'/name',oList.Response.MobileApps[iIndex].MobileName);
				storeParameter('devices/' + iIndex +'/uuid',oList.Response.MobileApps[iIndex].MobileAppUUID);
				storeParameter('devices/' + iIndex +'/appName',oList.Response.MobileApps[iIndex].AppName);
			}
			Property.store();
			var evt = new Event("event-push-notification.saved",{});			
			evt.raise();
		});
		
	} catch (e) {
		lWarn.logln('fetchDeviceList error ' + e);
	}
	
}


if (raisedEvent.name === 'event-push-notification') {
	Property.load();
	if (raisedEvent.parameter.actions=='fetchDeviceList') {
		fetchDeviceList();
	}
	if  (raisedEvent.parameter.actions=="execute") {
	    lInfo.logln('action : ' + raisedEvent.parameter.path);
        var sendingProperty = Property.getNode(raisedEvent.parameter.path);
		if (sendingProperty !=null) {
			sendPath(sendingProperty);
		} else {
			lWarn.logln('Infos zu Pfad nicht gefunden');
		}
        
	}
	if  (raisedEvent.parameter.actions=="test") {
	    lInfo.logln('action id: ' + raisedEvent.parameter.id);
        var sendingProperty = Property.getNode('values/' + raisedEvent.parameter.id);
		if (sendingProperty !=null) {
			sendPath(sendingProperty);
		} else {
			lWarn.logln('Infos zu Pfad nicht gefunden');
		}
        
	}

	if (raisedEvent.parameter.actions=="send") {
		var iDeviceNumber=raisedEvent.parameter.deviceIndex;
		var target=[]
		if (iDeviceNumber=='*') {	
			var oTargetProps=Property.getNode('devices').getChildren();
			target=[];
			for (var iIndex=0;iIndex<oTargetProps.length;iIndex++) {
				target.push(oTargetProps[iIndex].getChild('uuid').getValue());				
			}
		} else
			target=[Property.getNode('devices/' + iDeviceNumber + "/uuid").getValue()];
		var text=raisedEvent.parameter.text;
		cloudService.sendNotification(target,text);
	}

	if (raisedEvent.parameter.actions=='save')
	{
	    lInfo.logln("param value " + raisedEvent.parameter.value);		
		storeValue(raisedEvent.parameter.value);
		Property.store();
		var evt = new Event("event-push-notification.saved",{});			
		evt.raise();
	}
	if (raisedEvent.parameter.actions=='enable')
	{
	    lInfo.logln("enable value " + raisedEvent.parameter.id);	
	 
        var oIDObject=JSON.parse(unescape(raisedEvent.parameter.id));
        if (typeof(oIDObject)!='object')
            oIDObject=[oIDObject];

        for (var sKey in oIDObject) {
            var sanityCheck = Property.getNode('values/' + oIDObject[sKey]);
            if (sanityCheck != null) {                
        	    storeSubParameter(oIDObject[sKey] + '/conditions/enabled', true);
        		Property.store();
            }
        }
        
		var evt = new Event("event-push-notification.saved",{});			
		evt.raise();
	}
	if (raisedEvent.parameter.actions=='disable')
	{
	    lInfo.logln("event-mailer disable value " + raisedEvent.parameter.id);		

        var oIDObject=JSON.parse(unescape(raisedEvent.parameter.id));
        if (typeof(oIDObject)!='object')
            oIDObject=[oIDObject];

        for (var sKey in oIDObject) {
    	    var sanityCheck = Property.getNode('values/' + oIDObject[sKey]);
            if (sanityCheck != null) {
        	    storeSubParameter(oIDObject[sKey] + '/conditions/enabled', false);
        		Property.store();
            }        	
        }
	
        var evt = new Event("event-push-notification.saved",{});			
		evt.raise();
	}	
	if (raisedEvent.parameter.actions=='delete')
	{
	    lInfo.logln("delete value " + raisedEvent.parameter.id);		
		
        var oIDObject=JSON.parse(unescape(raisedEvent.parameter.id));
        if (typeof(oIDObject)!='object')
            oIDObject=[oIDObject];

        for (var sKey in oIDObject) {
    		var baseNode = Property.getNode('values');
    		if (baseNode!=null)
    			baseNode.removeChild(oIDObject[sKey]);    		
    		unregisterTrigger(baseNode.getPath()+"/" + oIDObject[sKey]);
    	}
		
		Property.store();
		var evt = new Event("event-push-notification.saved",{});			
		evt.raise();
	}		
} else if (raisedEvent.name == 'running') {
	lInfo.logln("Initializing App");		
	Property.load();
	checkAndUpdateData();
	updateAllTrigger();
	fetchDeviceList();
}
